@@ -1,39 +1,60 @@
+import phonenumbers
+from functools import wraps
 from sqlalchemy.orm import defer
-from sqlalchemy.orm.collections import InstrumentedList
+from sqlalchemy.orm.collections import InstrumentedList as _InstrumentedList
 from sqlalchemy.orm.mapper import Mapper
 from sqlalchemy.orm.query import _ColumnEntity
 from sqlalchemy.orm.properties import ColumnProperty
 from sqlalchemy.sql.expression import desc, asc
+from sqlalchemy import types
 
 
-class SmartList(InstrumentedList):
-    def has(self, attr):
-        """
-        Returns True if any member of this collection has given attribute
-        defined.
+class PhoneNumberType(types.TypeDecorator):
+    """
+    Changes PhoneNumber objects to a string representation on the way in and
+    changes them back to PhoneNumber objects on the way out. If E164 is used
+    as storing format, no country code is needed for parsing the database
+    value to PhoneNumber object.
+    """
+    STORE_FORMAT = phonenumbers.PhoneNumberFormat.E164
+    impl = types.Unicode(20)
 
-        Example syntax:
+    def __init__(self, country_code='US', max_length=20, *args, **kwargs):
+        super(PhoneNumberType, self).__init__(*args, **kwargs)
+        self.country_code = country_code
+        self.impl = types.Unicode(max_length)
 
-            >>> Category.articles.has('name')
+    def process_bind_param(self, value, dialect):
+        return phonenumbers.format_number(
+            value,
+            self.STORE_FORMAT
+        )
 
-        :param attr: collection member attribute name
-        """
-        adapter = self._sa_adapter
-        owner_class = adapter.owner_state.class_
-        relation = getattr(owner_class, adapter._key).property
-        relation_class = relation.mapper.class_
+    def process_result_value(self, value, dialect):
+        if self.STORE_FORMAT == phonenumbers.PhoneNumberFormat.E164:
+            return phonenumbers.parse(value)
+        return phonenumbers.parse(
+            value,
+            self.country_code
+        )
 
-        if not hasattr(relation_class, attr):
-            raise AttributeError(
-                'Class %s does not have attribute named %s' %
-                (relation_class.__name__, attr)
-            )
 
-        for record in self:
-            if getattr(record, attr):
-                return True
+class InstrumentedList(_InstrumentedList):
+    """Enhanced version of SQLAlchemy InstrumentedList. Provides some
+    additional functionality."""
 
-        return False
+    def any(self, attr):
+        return any(getattr(item, attr) for item in self)
+
+    def all(self, attr):
+        return all(getattr(item, attr) for item in self)
+
+
+def instrumented_list(f):
+    @wraps(f)
+    def wrapper(*args, **kwargs):
+        return InstrumentedList([item for item in f(*args, **kwargs)])
+    return wrapper
 
 
 def sort_query(query, sort):
